package javax.swing.text;

import java.awt.Component;
import java.awt.Container;
import javax.swing.event.DocumentEvent;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;

public class PlainView extends View implements TabExpander
{
    protected FontMetrics metrics;
    Element longLine;
    Font font;
    Segment lineBuffer;
    int tabSize;
    int tabBase;
    int sel0;
    int sel1;
    Color unselected;
    Color selected;
    int firstLineOffset;
    
    public PlainView(Element element) {
        super(element);
    }
    
    protected int getTabSize() {
        Integer n = (Integer)this.getDocument().getProperty("tabSize");
        return (n != null) ? ((int)n) : 8;
    }
    
    protected void drawLine(int i, Graphics graphics, int drawElement, int n) {
        Element element = this.getElement().getElement(i);
        try {
            if (element.isLeaf()) {
                this.drawElement(i, element, graphics, drawElement, n);
            }
            else {
                for (int elementCount = element.getElementCount(), j = 0; j < elementCount; ++j) {
                    drawElement = this.drawElement(i, element.getElement(j), graphics, drawElement, n);
                }
            }
        }
        catch (BadLocationException ex) {
            throw new StateInvariantError("Can't render line: " + i);
        }
    }
    
    private int drawElement(int n, Element element, Graphics graphics, int n2, int n3) throws BadLocationException {
        int startOffset = element.getStartOffset();
        int min = Math.min(this.getDocument().getLength(), element.getEndOffset());
        if (n == 0) {
            n2 += this.firstLineOffset;
        }
        AttributeSet attributes = element.getAttributes();
        if (Utilities.isComposedTextAttributeDefined(attributes)) {
            graphics.setColor(this.unselected);
            n2 = Utilities.drawComposedText(this, attributes, graphics, n2, n3, startOffset - element.getStartOffset(), min - element.getStartOffset());
        }
        else if (this.sel0 == this.sel1 || this.selected == this.unselected) {
            n2 = this.drawUnselectedText(graphics, n2, n3, startOffset, min);
        }
        else if (startOffset >= this.sel0 && startOffset <= this.sel1 && min >= this.sel0 && min <= this.sel1) {
            n2 = this.drawSelectedText(graphics, n2, n3, startOffset, min);
        }
        else if (this.sel0 >= startOffset && this.sel0 <= min) {
            if (this.sel1 >= startOffset && this.sel1 <= min) {
                n2 = this.drawUnselectedText(graphics, n2, n3, startOffset, this.sel0);
                n2 = this.drawSelectedText(graphics, n2, n3, this.sel0, this.sel1);
                n2 = this.drawUnselectedText(graphics, n2, n3, this.sel1, min);
            }
            else {
                n2 = this.drawUnselectedText(graphics, n2, n3, startOffset, this.sel0);
                n2 = this.drawSelectedText(graphics, n2, n3, this.sel0, min);
            }
        }
        else if (this.sel1 >= startOffset && this.sel1 <= min) {
            n2 = this.drawSelectedText(graphics, n2, n3, startOffset, this.sel1);
            n2 = this.drawUnselectedText(graphics, n2, n3, this.sel1, min);
        }
        else {
            n2 = this.drawUnselectedText(graphics, n2, n3, startOffset, min);
        }
        return n2;
    }
    
    protected int drawUnselectedText(Graphics graphics, int n, int n2, int n3, int n4) throws BadLocationException {
        graphics.setColor(this.unselected);
        Document document = this.getDocument();
        Segment sharedSegment = SegmentCache.getSharedSegment();
        document.getText(n3, n4 - n3, sharedSegment);
        int drawTabbedText = Utilities.drawTabbedText(this, sharedSegment, n, n2, graphics, this, n3);
        SegmentCache.releaseSharedSegment(sharedSegment);
        return drawTabbedText;
    }
    
    protected int drawSelectedText(Graphics graphics, int n, int n2, int n3, int n4) throws BadLocationException {
        graphics.setColor(this.selected);
        Document document = this.getDocument();
        Segment sharedSegment = SegmentCache.getSharedSegment();
        document.getText(n3, n4 - n3, sharedSegment);
        int drawTabbedText = Utilities.drawTabbedText(this, sharedSegment, n, n2, graphics, this, n3);
        SegmentCache.releaseSharedSegment(sharedSegment);
        return drawTabbedText;
    }
    
    protected final Segment getLineBuffer() {
        if (this.lineBuffer == null) {
            this.lineBuffer = new Segment();
        }
        return this.lineBuffer;
    }
    
    protected void updateMetrics() {
        if (this.font != this.getContainer().getFont()) {
            this.calculateLongestLine();
            this.tabSize = this.getTabSize() * this.metrics.charWidth('m');
        }
    }
    
    @Override
    public float getPreferredSpan(int i) {
        this.updateMetrics();
        switch (i) {
            case 0: {
                return (float)this.getLineWidth(this.longLine);
            }
            case 1: {
                return (float)(this.getElement().getElementCount() * this.metrics.getHeight());
            }
            default: {
                throw new IllegalArgumentException("Invalid axis: " + i);
            }
        }
    }
    
    @Override
    public void paint(Graphics graphics, Shape adjustPaintRegion) {
        adjustPaintRegion = this.adjustPaintRegion(adjustPaintRegion);
        Rectangle rectangle = (Rectangle)adjustPaintRegion;
        this.tabBase = rectangle.x;
        JTextComponent textComponent = (JTextComponent)this.getContainer();
        Highlighter highlighter = textComponent.getHighlighter();
        graphics.setFont(textComponent.getFont());
        this.sel0 = textComponent.getSelectionStart();
        this.sel1 = textComponent.getSelectionEnd();
        this.unselected = (textComponent.isEnabled() ? textComponent.getForeground() : textComponent.getDisabledTextColor());
        this.selected = ((textComponent.getCaret().isSelectionVisible() && highlighter != null) ? textComponent.getSelectedTextColor() : this.unselected);
        this.updateMetrics();
        Rectangle clipBounds = graphics.getClipBounds();
        int height = this.metrics.getHeight();
        int n = rectangle.y + rectangle.height - (clipBounds.y + clipBounds.height);
        int n2 = clipBounds.y - rectangle.y;
        int n3;
        int max;
        int i;
        if (height > 0) {
            max = Math.max(0, n / height);
            i = Math.max(0, n2 / height);
            n3 = rectangle.height / height;
            if (rectangle.height % height != 0) {
                ++n3;
            }
        }
        else {
            i = (max = (n3 = 0));
        }
        Rectangle lineToRect = this.lineToRect(adjustPaintRegion, i);
        int n4 = lineToRect.y + this.metrics.getAscent();
        int x = lineToRect.x;
        Element element = this.getElement();
        int elementCount = element.getElementCount();
        int min = Math.min(elementCount, n3 - max);
        --elementCount;
        LayeredHighlighter layeredHighlighter = (highlighter instanceof LayeredHighlighter) ? ((LayeredHighlighter)highlighter) : null;
        for (; i < min; ++i) {
            if (layeredHighlighter != null) {
                Element element2 = element.getElement(i);
                if (i == elementCount) {
                    layeredHighlighter.paintLayeredHighlights(graphics, element2.getStartOffset(), element2.getEndOffset(), adjustPaintRegion, textComponent, this);
                }
                else {
                    layeredHighlighter.paintLayeredHighlights(graphics, element2.getStartOffset(), element2.getEndOffset() - 1, adjustPaintRegion, textComponent, this);
                }
            }
            this.drawLine(i, graphics, x, n4);
            n4 += height;
            if (i == 0) {
                x -= this.firstLineOffset;
            }
        }
    }
    
    Shape adjustPaintRegion(Shape shape) {
        return shape;
    }
    
    @Override
    public Shape modelToView(int n, Shape shape, Position.Bias bias) throws BadLocationException {
        Document document = this.getDocument();
        Element element = this.getElement();
        int elementIndex = element.getElementIndex(n);
        if (elementIndex < 0) {
            return this.lineToRect(shape, 0);
        }
        Rectangle lineToRect = this.lineToRect(shape, elementIndex);
        this.tabBase = lineToRect.x;
        int startOffset = element.getElement(elementIndex).getStartOffset();
        Segment sharedSegment = SegmentCache.getSharedSegment();
        document.getText(startOffset, n - startOffset, sharedSegment);
        int tabbedTextWidth = Utilities.getTabbedTextWidth(sharedSegment, this.metrics, this.tabBase, this, startOffset);
        SegmentCache.releaseSharedSegment(sharedSegment);
        lineToRect.x += tabbedTextWidth;
        lineToRect.width = 1;
        lineToRect.height = this.metrics.getHeight();
        return lineToRect;
    }
    
    @Override
    public int viewToModel(float n, float n2, Shape shape, Position.Bias[] array) {
        array[0] = Position.Bias.Forward;
        Rectangle bounds = shape.getBounds();
        Document document = this.getDocument();
        int n3 = (int)n;
        int n4 = (int)n2;
        if (n4 < bounds.y) {
            return this.getStartOffset();
        }
        if (n4 > bounds.y + bounds.height) {
            return this.getEndOffset() - 1;
        }
        Element defaultRootElement = document.getDefaultRootElement();
        int height = this.metrics.getHeight();
        int n5 = (height > 0) ? Math.abs((n4 - bounds.y) / height) : (defaultRootElement.getElementCount() - 1);
        if (n5 >= defaultRootElement.getElementCount()) {
            return this.getEndOffset() - 1;
        }
        Element element = defaultRootElement.getElement(n5);
        if (n5 == 0) {
            bounds.x += this.firstLineOffset;
            bounds.width -= this.firstLineOffset;
        }
        if (n3 < bounds.x) {
            return element.getStartOffset();
        }
        if (n3 > bounds.x + bounds.width) {
            return element.getEndOffset() - 1;
        }
        try {
            int startOffset = element.getStartOffset();
            int n6 = element.getEndOffset() - 1;
            Segment sharedSegment = SegmentCache.getSharedSegment();
            document.getText(startOffset, n6 - startOffset, sharedSegment);
            this.tabBase = bounds.x;
            int n7 = startOffset + Utilities.getTabbedTextOffset(sharedSegment, this.metrics, this.tabBase, n3, this, startOffset);
            SegmentCache.releaseSharedSegment(sharedSegment);
            return n7;
        }
        catch (BadLocationException ex) {
            return -1;
        }
    }
    
    @Override
    public void insertUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.updateDamage(documentEvent, shape, viewFactory);
    }
    
    @Override
    public void removeUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.updateDamage(documentEvent, shape, viewFactory);
    }
    
    @Override
    public void changedUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.updateDamage(documentEvent, shape, viewFactory);
    }
    
    @Override
    public void setSize(float n, float n2) {
        super.setSize(n, n2);
        this.updateMetrics();
    }
    
    @Override
    public float nextTabStop(float n, int n2) {
        if (this.tabSize == 0) {
            return n;
        }
        return (float)(this.tabBase + (((int)n - this.tabBase) / this.tabSize + 1) * this.tabSize);
    }
    
    protected void updateDamage(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        Container container = this.getContainer();
        this.updateMetrics();
        DocumentEvent.ElementChange change = documentEvent.getChange(this.getElement());
        Element[] array = (change != null) ? change.getChildrenAdded() : null;
        Element[] array2 = (change != null) ? change.getChildrenRemoved() : null;
        if ((array != null && array.length > 0) || (array2 != null && array2.length > 0)) {
            if (array != null) {
                int lineWidth2 = this.getLineWidth(this.longLine);
                for (int i = 0; i < array.length; ++i) {
                    int lineWidth3 = this.getLineWidth(array[i]);
                    if (lineWidth3 > lineWidth2) {
                        lineWidth2 = lineWidth3;
                        this.longLine = array[i];
                    }
                }
            }
            if (array2 != null) {
                int j = 0;
                while (j < array2.length) {
                    if (array2[j] == this.longLine) {
                        this.calculateLongestLine();
                        break;
                    }
                    ++j;
                }
            }
            this.preferenceChanged(null, true, true);
            container.repaint();
        }
        else {
            Element element = this.getElement();
            int elementIndex = element.getElementIndex(documentEvent.getOffset());
            this.damageLineRange(elementIndex, elementIndex, shape, container);
            if (documentEvent.getType() == DocumentEvent.EventType.INSERT) {
                int lineWidth = this.getLineWidth(this.longLine);
                Element element2 = element.getElement(elementIndex);
                if (element2 == this.longLine) {
                    this.preferenceChanged(null, true, false);
                }
                else if (this.getLineWidth(element2) > lineWidth) {
                    this.longLine = element2;
                    this.preferenceChanged(null, true, false);
                }
            }
            else if (documentEvent.getType() == DocumentEvent.EventType.REMOVE && element.getElement(elementIndex) == this.longLine) {
                this.calculateLongestLine();
                this.preferenceChanged(null, true, false);
            }
        }
    }
    
    protected void damageLineRange(int n, int n2, Shape shape, Component component) {
        if (shape != null) {
            Rectangle lineToRect = this.lineToRect(shape, n);
            Rectangle lineToRect2 = this.lineToRect(shape, n2);
            if (lineToRect != null && lineToRect2 != null) {
                Rectangle union = lineToRect.union(lineToRect2);
                component.repaint(union.x, union.y, union.width, union.height);
            }
            else {
                component.repaint();
            }
        }
    }
    
    protected Rectangle lineToRect(Shape shape, int n) {
        Rectangle rectangle = null;
        this.updateMetrics();
        if (this.metrics != null) {
            Rectangle bounds = shape.getBounds();
            if (n == 0) {
                bounds.x += this.firstLineOffset;
                bounds.width -= this.firstLineOffset;
            }
            rectangle = new Rectangle(bounds.x, bounds.y + n * this.metrics.getHeight(), bounds.width, this.metrics.getHeight());
            return rectangle;
        }
        else {
            return rectangle;
        }
    }
    
    private void calculateLongestLine() {
        Container container = this.getContainer();
        this.font = container.getFont();
        this.metrics = container.getFontMetrics(this.font);
        this.getDocument();
        Element element = this.getElement();
        int elementCount = element.getElementCount();
        int n = -1;
        for (int i = 0; i < elementCount; ++i) {
            Element element2 = element.getElement(i);
            int lineWidth = this.getLineWidth(element2);
            if (lineWidth > n) {
                n = lineWidth;
                this.longLine = element2;
            }
        }
    }
    
    private int getLineWidth(Element element) {
        if (element == null) {
            return 0;
        }
        int startOffset = element.getStartOffset();
        int endOffset = element.getEndOffset();
        Segment sharedSegment = SegmentCache.getSharedSegment();
        int tabbedTextWidth;
        try {
            element.getDocument().getText(startOffset, endOffset - startOffset, sharedSegment);
            tabbedTextWidth = Utilities.getTabbedTextWidth(sharedSegment, this.metrics, this.tabBase, this, startOffset);
        }
        catch (BadLocationException ex) {
            tabbedTextWidth = 0;
        }
        SegmentCache.releaseSharedSegment(sharedSegment);
        return tabbedTextWidth;
    }
}
